@@ -26,16 +26,17 @@
 
 /**
  * SECTION:element-opusdec
+ * @title: opusdec
  * @see_also: opusenc, oggdemux
  *
  * This element decodes a OPUS stream to raw integer audio.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v filesrc location=opus.ogg ! oggdemux ! opusdec ! audioconvert ! audioresample ! alsasink
- * ]| Decode an Ogg/Opus file. To create an Ogg/Opus file refer to the documentation of opusenc.
- * </refsect2>
+ * ]|
+ * Decode an Ogg/Opus file. To create an Ogg/Opus file refer to the documentation of opusenc.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +46,7 @@
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
+#include "gstopuselements.h"
 #include "gstopusheader.h"
 #include "gstopuscommon.h"
 #include "gstopusdec.h"
@@ -77,17 +79,22 @@ static GstStaticPadTemplate opus_dec_sink_factory =
     );
 
 G_DEFINE_TYPE (GstOpusDec, gst_opus_dec, GST_TYPE_AUDIO_DECODER);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (opusdec, "opusdec",
+    GST_RANK_PRIMARY, GST_TYPE_OPUS_DEC, opus_element_init (plugin));
 
 #define DB_TO_LINEAR(x) pow (10., (x) / 20.)
 
 #define DEFAULT_USE_INBAND_FEC FALSE
 #define DEFAULT_APPLY_GAIN TRUE
+#define DEFAULT_PHASE_INVERSION FALSE
 
 enum
 {
   PROP_0,
   PROP_USE_INBAND_FEC,
-  PROP_APPLY_GAIN
+  PROP_APPLY_GAIN,
+  PROP_PHASE_INVERSION,
+  PROP_STATS,
 };
 
 
@@ -131,7 +138,7 @@ gst_opus_dec_class_init (GstOpusDecClass * klass)
   gst_element_class_add_static_pad_template (element_class,
       &opus_dec_sink_factory);
   gst_element_class_set_static_metadata (element_class, "Opus audio decoder",
-      "Codec/Decoder/Audio", "decode opus streams to audio",
+      "Codec/Decoder/Audio/Converter", "decode opus streams to audio",
       "Vincent Penquerc'h <vincent.penquerch@collabora.co.uk>");
   g_object_class_install_property (gobject_class, PROP_USE_INBAND_FEC,
       g_param_spec_boolean ("use-inband-fec", "Use in-band FEC",
@@ -143,6 +150,39 @@ gst_opus_dec_class_init (GstOpusDecClass * klass)
           "Apply gain if any is specified in the header", DEFAULT_APPLY_GAIN,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+#ifdef OPUS_SET_PHASE_INVERSION_DISABLED_REQUEST
+  g_object_class_install_property (gobject_class, PROP_PHASE_INVERSION,
+      g_param_spec_boolean ("phase-inversion",
+          "Control Phase Inversion", "Set to true to enable phase inversion, "
+          "this will slightly improve stereo quality, but will have side "
+          "effects when downmixed to mono.", DEFAULT_PHASE_INVERSION,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+#endif
+
+  /**
+   * GstOpusDec:stats:
+   *
+   * Various decoder statistics. This property returns a GstStructure
+   * with name application/x-opusdec-stats with the following fields:
+   *
+   * * #guint64 `num-pushed`: the number of packets pushed out.
+   * * #guint64 `num-gap`: the number of gap packets received.
+   * * #guint64 `plc-num-samples`: the number of samples generated using PLC
+   * * #guint64 `plc-duration`: the total duration, in ns, of samples generated using PLC
+   * * #guint32 `bandwidth`: decoder last bandpass, in kHz, or 0 if unknown
+   * * #guint32 `sample-rate`: decoder sampling rate, or 0 if unknown
+   * * #guint32 `gain`: decoder gain adjustement, in Q8 dB units, or 0 if unknown
+   * * #guint32 `last-packet-duration`: duration, in samples, of the last packet successfully decoded or concealed, or 0 if unknown
+   * * #guint `channels`: the number of channels
+   *
+   * Since: 1.18
+   */
+  g_object_class_install_property (gobject_class, PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics",
+          "Various statistics", GST_TYPE_STRUCTURE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   GST_DEBUG_CATEGORY_INIT (opusdec_debug, "opusdec", 0,
       "opus decoding element");
 }
@@ -174,6 +214,7 @@ gst_opus_dec_init (GstOpusDec * dec)
 {
   dec->use_inband_fec = FALSE;
   dec->apply_gain = DEFAULT_APPLY_GAIN;
+  dec->phase_inversion = DEFAULT_PHASE_INVERSION;
 
   gst_audio_decoder_set_needs_format (GST_AUDIO_DECODER (dec), TRUE);
   gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
@@ -201,6 +242,13 @@ gst_opus_dec_start (GstAudioDecoder * dec)
     gst_audio_decoder_set_latency (dec, 120 * GST_MSECOND, 120 * GST_MSECOND);
   }
 
+  GST_OBJECT_LOCK (dec);
+  odec->num_pushed = 0;
+  odec->num_gap = 0;
+  odec->plc_num_samples = 0;
+  odec->plc_duration = 0;
+  GST_OBJECT_UNLOCK (dec);
+
   return TRUE;
 }
 
@@ -262,7 +310,7 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
     gst_structure_fixate_field_nearest_int (s, "rate", dec->sample_rate);
     gst_structure_get_int (s, "rate", &rate);
     channels = dec->n_channels > 0 ? dec->n_channels : 2;
-    gst_structure_fixate_field_nearest_int (s, "channels", dec->n_channels);
+    gst_structure_fixate_field_nearest_int (s, "channels", channels);
     gst_structure_get_int (s, "channels", &channels);
 
     gst_caps_unref (inter);
@@ -315,6 +363,7 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
 {
   GstAudioChannelPosition pos[64];
   const GstAudioChannelPosition *posn = NULL;
+  guint8 n_channels;
 
   if (!gst_opus_header_is_id_header (buf)) {
     GST_ELEMENT_ERROR (dec, STREAM, FORMAT, (NULL),
@@ -324,7 +373,7 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
 
   if (!gst_codec_utils_opus_parse_header (buf,
           &dec->sample_rate,
-          &dec->n_channels,
+          &n_channels,
           &dec->channel_mapping_family,
           &dec->n_streams,
           &dec->n_stereo_streams,
@@ -333,6 +382,7 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
         ("Failed to parse Opus ID header"));
     return GST_FLOW_ERROR;
   }
+  dec->n_channels = n_channels;
   dec->r128_gain_volume = gst_opus_dec_get_r128_volume (dec->r128_gain);
 
   GST_INFO_OBJECT (dec,
@@ -497,6 +547,20 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
         dec->n_streams, dec->n_stereo_streams, dec->channel_mapping, &err);
     if (!dec->state || err != OPUS_OK)
       goto creation_failed;
+
+#ifdef OPUS_SET_PHASE_INVERSION_DISABLED_REQUEST
+    {
+      int err;
+      err = opus_multistream_decoder_ctl (dec->state,
+          OPUS_SET_PHASE_INVERSION_DISABLED (!dec->phase_inversion));
+      if (err != OPUS_OK)
+        GST_WARNING_OBJECT (dec, "Could not configure phase inversion: %s",
+            opus_strerror (err));
+    }
+#else
+    GST_WARNING_OBJECT (dec, "Phase inversion request is not support by this "
+        "version of the Opus Library");
+#endif
   }
 
   if (buffer) {
@@ -558,6 +622,10 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
         " plus leftover %" GST_TIME_FORMAT, GST_TIME_ARGS (missing_duration),
         GST_TIME_ARGS (dec->leftover_plc_duration));
 
+    GST_OBJECT_LOCK (dec);
+    dec->num_gap++;
+    GST_OBJECT_UNLOCK (dec);
+
     /* add the leftover PLC duration to that of the buffer */
     missing_duration += dec->leftover_plc_duration;
 
@@ -590,6 +658,11 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
         " num frame samples: %d new leftover: %" GST_TIME_FORMAT,
         GST_TIME_ARGS (aligned_missing_duration), samples,
         GST_TIME_ARGS (dec->leftover_plc_duration));
+
+    GST_OBJECT_LOCK (dec);
+    dec->plc_num_samples += samples;
+    dec->plc_duration += aligned_missing_duration;
+    GST_OBJECT_UNLOCK (dec);
   } else {
     /* use maximum size (120 ms) as the number of returned samples is
        not constant over the stream. */
@@ -738,6 +811,10 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     gst_buffer_replace (&dec->last_buffer, buffer);
   }
 
+  GST_OBJECT_LOCK (dec);
+  dec->num_pushed++;
+  GST_OBJECT_UNLOCK (dec);
+
   res = gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (dec), outbuf, 1);
 
   if (res != GST_FLOW_OK)
@@ -811,13 +888,15 @@ gst_opus_dec_set_format (GstAudioDecoder * bdec, GstCaps * caps)
     }
   } else {
     const GstAudioChannelPosition *posn = NULL;
+    guint8 n_channels;
 
     if (!gst_codec_utils_opus_parse_caps (caps, &dec->sample_rate,
-            &dec->n_channels, &dec->channel_mapping_family, &dec->n_streams,
-            &dec->n_stereo_streams, dec->channel_mapping)) {
+            &n_channels, &dec->channel_mapping_family,
+            &dec->n_streams, &dec->n_stereo_streams, dec->channel_mapping)) {
       ret = FALSE;
       goto done;
     }
+    dec->n_channels = n_channels;
 
     if (dec->channel_mapping_family == 1 && dec->n_channels <= 8)
       posn = gst_opus_channel_positions[dec->n_channels - 1];
@@ -915,6 +994,126 @@ gst_opus_dec_handle_frame (GstAudioDecoder * adec, GstBuffer * buf)
   return res;
 }
 
+/* Called with object lock hold */
+static guint32
+get_bandwidth (GstOpusDec * self)
+{
+  gint err;
+  gint32 bw;
+
+  if (!self->state)
+    return 0;
+
+  err = opus_multistream_decoder_ctl (self->state, OPUS_GET_BANDWIDTH (&bw));
+  if (err != OPUS_OK) {
+    GST_WARNING_OBJECT (self, "Could not retrieve bandwith: %s",
+        opus_strerror (err));
+    return 0;
+  }
+
+  switch (bw) {
+    case OPUS_BANDWIDTH_NARROWBAND:
+      return 4;
+    case OPUS_BANDWIDTH_MEDIUMBAND:
+      return 6;
+    case OPUS_BANDWIDTH_WIDEBAND:
+      return 8;
+    case OPUS_BANDWIDTH_SUPERWIDEBAND:
+      return 12;
+    case OPUS_BANDWIDTH_FULLBAND:
+      return 20;
+    default:
+      GST_WARNING_OBJECT (self, "Unknown bandwith enum: %d", bw);
+      return 0;
+  }
+}
+
+/* Called with object lock hold */
+static guint32
+get_sample_rate (GstOpusDec * self)
+{
+  gint err;
+  gint32 rate;
+
+  if (!self->state)
+    return 0;
+
+  err =
+      opus_multistream_decoder_ctl (self->state, OPUS_GET_SAMPLE_RATE (&rate));
+  if (err != OPUS_OK) {
+    GST_WARNING_OBJECT (self, "Could not retrieve sample rate: %s",
+        opus_strerror (err));
+    return 0;
+  }
+
+  return rate;
+}
+
+/* Called with object lock hold */
+static guint32
+get_gain (GstOpusDec * self)
+{
+  gint err;
+  gint32 gain;
+
+  if (!self->state)
+    return 0;
+
+  err = opus_multistream_decoder_ctl (self->state, OPUS_GET_GAIN (&gain));
+  if (err != OPUS_OK) {
+    GST_WARNING_OBJECT (self, "Could not retrieve gain: %s",
+        opus_strerror (err));
+    return 0;
+  }
+
+  return gain;
+}
+
+/* Called with object lock hold */
+static guint32
+get_last_packet_duration (GstOpusDec * self)
+{
+  gint err;
+  gint32 duration;
+
+  if (!self->state)
+    return 0;
+
+  err =
+      opus_multistream_decoder_ctl (self->state,
+      OPUS_GET_LAST_PACKET_DURATION (&duration));
+  if (err != OPUS_OK) {
+    GST_WARNING_OBJECT (self, "Could not retrieve last packet duration: %s",
+        opus_strerror (err));
+    return 0;
+  }
+
+  return duration;
+}
+
+static GstStructure *
+gst_opus_dec_create_stats (GstOpusDec * self)
+{
+  GstStructure *s;
+
+  GST_OBJECT_LOCK (self);
+
+  s = gst_structure_new ("application/x-opusdec-stats",
+      "num-pushed", G_TYPE_UINT64, self->num_pushed,
+      "num-gap", G_TYPE_UINT64, self->num_gap,
+      "plc-num-samples", G_TYPE_UINT64, self->plc_num_samples,
+      "plc-duration", G_TYPE_UINT64, self->plc_duration,
+      "bandwidth", G_TYPE_UINT, get_bandwidth (self),
+      "sample-rate", G_TYPE_UINT, get_sample_rate (self),
+      "gain", G_TYPE_UINT, get_gain (self),
+      "last-packet-duration", G_TYPE_UINT, get_last_packet_duration (self),
+      "channels", G_TYPE_UINT, self->n_channels, NULL);
+
+  GST_OBJECT_UNLOCK (self);
+
+  return s;
+}
+
 static void
 gst_opus_dec_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
@@ -928,6 +1127,12 @@ gst_opus_dec_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_APPLY_GAIN:
       g_value_set_boolean (value, dec->apply_gain);
       break;
+    case PROP_PHASE_INVERSION:
+      g_value_set_boolean (value, dec->phase_inversion);
+      break;
+    case PROP_STATS:
+      g_value_take_boxed (value, gst_opus_dec_create_stats (dec));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -947,6 +1152,9 @@ gst_opus_dec_set_property (GObject * object, guint prop_id,
     case PROP_APPLY_GAIN:
       dec->apply_gain = g_value_get_boolean (value);
       break;
+    case PROP_PHASE_INVERSION:
+      dec->phase_inversion = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
