@@ -33,21 +33,21 @@
 
 G_BEGIN_DECLS
 
-GST_EXPORT
+GST_GL_API
 GType gst_gl_overlay_compositor_get_type (void);
 
 /**
- * GstGLOverlayCompositor
+ * GstGLOverlayCompositor:
  *
  * Opaque #GstGLOverlayCompositor object
  */
 struct _GstGLOverlayCompositor
 {
+  /*< private >*/
   GstObject parent;
 
   GstGLContext *context;
 
-  /* <private> */
   guint last_window_width;
   guint last_window_height;
 
@@ -68,24 +68,24 @@ struct _GstGLOverlayCompositorClass
 {
   GstObjectClass object_class;
 
-  /* <private> */
+  /*< private >*/
   gpointer _padding[GST_PADDING];
 };
 
-GST_EXPORT
+GST_GL_API
 GstGLOverlayCompositor *gst_gl_overlay_compositor_new (GstGLContext * context);
 
-GST_EXPORT
+GST_GL_API
 void gst_gl_overlay_compositor_free_overlays (GstGLOverlayCompositor * compositor);
 
-GST_EXPORT
+GST_GL_API
 void gst_gl_overlay_compositor_upload_overlays (GstGLOverlayCompositor * compositor,
         GstBuffer * buf);
 
-GST_EXPORT
+GST_GL_API
 void gst_gl_overlay_compositor_draw_overlays (GstGLOverlayCompositor * compositor);
 
-GST_EXPORT
+GST_GL_API
 GstCaps * gst_gl_overlay_compositor_add_caps(GstCaps * caps);
 
 G_END_DECLS
