@@ -339,6 +339,36 @@ gst_rtsp_url_get_request_uri (const GstRTSPUrl * url)
     uri = g_strdup_printf ("rtsp://%s%s%s%s%s%s", pre_host, url->host,
         post_host, url->abspath, pre_query, query);
   }
+  return uri;
+}
+
+/**
+ * gst_rtsp_url_get_request_uri_with_control:
+ * @url: a #GstRTSPUrl
+ * @control_path: an RTSP aggregate control path
+ *
+ * Get a newly allocated string describing the request URI for @url
+ * combined with the control path for @control_path
+ *
+ * Returns: a string with the request URI combined with the control path.
+ * g_free() after usage.
+ *
+ * Since: 1.18
+ */
+gchar *
+gst_rtsp_url_get_request_uri_with_control (const GstRTSPUrl * url,
+    const gchar * control_path)
+{
+  gchar *url_string;
+  gchar *uri;
+
+  g_return_val_if_fail (url != NULL, NULL);
+
+  /* FIXME: Use GUri here once we can depend on GLib 2.66 */
+
+  url_string = gst_rtsp_url_get_request_uri (url);
+  uri = gst_uri_join_strings (url_string, control_path);
+  g_clear_pointer (&url_string, g_free);
 
   return uri;
 }
