@@ -56,41 +56,16 @@
 #include <gst/video/navigation.h>
 #include <gst/video/video-enumtypes.h>
 
-static void gst_navigation_class_init (GstNavigationInterface * iface);
-
 #define GST_NAVIGATION_MESSAGE_NAME "GstNavigationMessage"
 #define GST_NAVIGATION_QUERY_NAME "GstNavigationQuery"
 #define GST_NAVIGATION_EVENT_NAME "application/x-gst-navigation"
 
 #define WARN_IF_FAIL(exp,msg) if(G_UNLIKELY(!(exp))){g_warning("%s",(msg));}
 
-GType
-gst_navigation_get_type (void)
-{
-  static GType gst_navigation_type = 0;
-
-  if (!gst_navigation_type) {
-    static const GTypeInfo gst_navigation_info = {
-      sizeof (GstNavigationInterface),
-      (GBaseInitFunc) gst_navigation_class_init,
-      NULL,
-      NULL,
-      NULL,
-      NULL,
-      0,
-      0,
-      NULL,
-    };
-
-    gst_navigation_type = g_type_register_static (G_TYPE_INTERFACE,
-        "GstNavigation", &gst_navigation_info, 0);
-  }
-
-  return gst_navigation_type;
-}
+G_DEFINE_INTERFACE (GstNavigation, gst_navigation, 0);
 
 static void
-gst_navigation_class_init (GstNavigationInterface * iface)
+gst_navigation_default_init (GstNavigationInterface * iface)
 {
   /* default virtual functions */
   iface->send_event = NULL;
@@ -122,6 +97,9 @@ void
 gst_navigation_send_key_event (GstNavigation * navigation, const char *event,
     const char *key)
 {
+  g_return_if_fail (g_strcmp0 (event, "key-press") == 0 ||
+      g_strcmp0 (event, "key-release") == 0);
+
   gst_navigation_send_event (navigation,
       gst_structure_new (GST_NAVIGATION_EVENT_NAME, "event", G_TYPE_STRING,
           event, "key", G_TYPE_STRING, key, NULL));
@@ -147,12 +125,44 @@ void
 gst_navigation_send_mouse_event (GstNavigation * navigation, const char *event,
     int button, double x, double y)
 {
+  g_return_if_fail (g_strcmp0 (event, "mouse-button-press") == 0 ||
+      g_strcmp0 (event, "mouse-button-release") == 0 ||
+      g_strcmp0 (event, "mouse-move") == 0);
+
   gst_navigation_send_event (navigation,
       gst_structure_new (GST_NAVIGATION_EVENT_NAME, "event", G_TYPE_STRING,
           event, "button", G_TYPE_INT, button, "pointer_x", G_TYPE_DOUBLE, x,
           "pointer_y", G_TYPE_DOUBLE, y, NULL));
 }
 
+/**
+ * gst_navigation_send_mouse_scroll_event:
+ * @navigation: The navigation interface instance
+ * @x: The x coordinate of the mouse event.
+ * @y: The y coordinate of the mouse event.
+ * @delta_x: The delta_x coordinate of the mouse event.
+ * @delta_y: The delta_y coordinate of the mouse event.
+ *
+ * Sends a mouse scroll event to the navigation interface. Mouse event coordinates
+ * are sent relative to the display space of the related output area. This is
+ * usually the size in pixels of the window associated with the element
+ * implementing the #GstNavigation interface.
+ *
+ * Since: 1.18
+ */
+void
+gst_navigation_send_mouse_scroll_event (GstNavigation * navigation,
+    double x, double y, double delta_x, double delta_y)
+{
+  gst_navigation_send_event (navigation,
+      gst_structure_new (GST_NAVIGATION_EVENT_NAME,
+          "event", G_TYPE_STRING, "mouse-scroll",
+          "pointer_x", G_TYPE_DOUBLE, x,
+          "pointer_y", G_TYPE_DOUBLE, y,
+          "delta_pointer_x", G_TYPE_DOUBLE, delta_x,
+          "delta_pointer_y", G_TYPE_DOUBLE, delta_y, NULL));
+}
+
 /**
  * gst_navigation_send_command:
  * @navigation: The navigation interface instance
@@ -277,7 +287,8 @@ gst_navigation_query_set_commands (GstQuery * query, gint n_cmds, ...)
  * gst_navigation_query_set_commandsv:
  * @query: a #GstQuery
  * @n_cmds: the number of commands to set.
- * @cmds: An array containing @n_cmds @GstNavigationCommand values.
+ * @cmds: (array length=n_cmds): An array containing @n_cmds
+ *     @GstNavigationCommand values.
  *
  * Set the #GstNavigation command query result fields in @query. The number
  * of commands passed must be equal to @n_commands.
@@ -303,7 +314,7 @@ gst_navigation_query_set_commandsv (GstQuery * query, gint n_cmds,
 /**
  * gst_navigation_query_parse_commands_length:
  * @query: a #GstQuery
- * @n_cmds: (out): the number of commands in this query.
+ * @n_cmds: (out) (optional): the number of commands in this query.
  *
  * Parse the number of commands in the #GstNavigation commands @query.
  *
@@ -334,7 +345,7 @@ gst_navigation_query_parse_commands_length (GstQuery * query, guint * n_cmds)
  * gst_navigation_query_parse_commands_nth:
  * @query: a #GstQuery
  * @nth: the nth command to retrieve.
- * @cmd: (out): a pointer to store the nth command into.
+ * @cmd: (out) (optional): a pointer to store the nth command into.
  *
  * Parse the #GstNavigation command query and retrieve the @nth command from
  * it into @cmd. If the list contains less elements than @nth, @cmd will be
@@ -415,10 +426,10 @@ gst_navigation_query_set_angles (GstQuery * query, guint cur_angle,
 /**
  * gst_navigation_query_parse_angles:
  * @query: a #GstQuery
- * @cur_angle: Pointer to a #guint into which to store the currently selected
- * angle value from the query, or NULL
- * @n_angles: Pointer to a #guint into which to store the number of angles
- * value from the query, or NULL
+ * @cur_angle: (out) (optional): Pointer to a #guint into which to store the
+ *     currently selected angle value from the query, or NULL
+ * @n_angles: (out) (optional): Pointer to a #guint into which to store the
+ *     number of angles value from the query, or NULL
  *
  * Parse the current angle number in the #GstNavigation angles @query into the
  * #guint pointed to by the @cur_angle variable, and the number of available
@@ -522,8 +533,8 @@ gst_navigation_message_new_mouse_over (GstObject * src, gboolean active)
 /**
  * gst_navigation_message_parse_mouse_over:
  * @message: A #GstMessage to inspect.
- * @active: A pointer to a gboolean to receive the active/inactive state,
- * or NULL.
+ * @active: (out) (optional): A pointer to a gboolean to receive the
+ *     active/inactive state, or NULL.
  *
  * Parse a #GstNavigation message of type #GST_NAVIGATION_MESSAGE_MOUSE_OVER
  * and extract the active/inactive flag. If the mouse over event is marked
@@ -576,8 +587,8 @@ gst_navigation_message_new_event (GstObject * src, GstEvent * event)
 /**
  * gst_navigation_message_parse_event:
  * @message: A #GstMessage to inspect.
- * @event: (out) (transfer full): a pointer to a #GstEvent to receive the
- *     contained navigation event.
+ * @event: (out) (optional) (transfer full): a pointer to a #GstEvent to receive
+ *     the contained navigation event.
  *
  * Parse a #GstNavigation message of type #GST_NAVIGATION_MESSAGE_EVENT
  * and extract contained #GstEvent. The caller must unref the @event when done
@@ -657,9 +668,10 @@ gst_navigation_message_new_angles_changed (GstObject * src, guint cur_angle,
 /**
  * gst_navigation_message_parse_angles_changed:
  * @message: A #GstMessage to inspect.
- * @cur_angle: A pointer to a #guint to receive the new current angle number,
- * or NULL
- * @n_angles: A pointer to a #guint to receive the new angle count, or NULL.
+ * @cur_angle: (out) (optional): A pointer to a #guint to receive the new
+ *     current angle number, or NULL
+ * @n_angles: (out) (optional): A pointer to a #guint to receive the new angle
+ *     count, or NULL.
  *
  * Parse a #GstNavigation message of type GST_NAVIGATION_MESSAGE_ANGLES_CHANGED
  * and extract the @cur_angle and @n_angles parameters.
@@ -721,6 +733,8 @@ gst_navigation_event_get_type (GstEvent * event)
     return GST_NAVIGATION_EVENT_MOUSE_BUTTON_RELEASE;
   else if (g_str_equal (e_type, "mouse-move"))
     return GST_NAVIGATION_EVENT_MOUSE_MOVE;
+  else if (g_str_equal (e_type, "mouse-scroll"))
+    return GST_NAVIGATION_EVENT_MOUSE_SCROLL;
   else if (g_str_equal (e_type, "key-press"))
     return GST_NAVIGATION_EVENT_KEY_PRESS;
   else if (g_str_equal (e_type, "key-release"))
@@ -734,9 +748,9 @@ gst_navigation_event_get_type (GstEvent * event)
 /**
  * gst_navigation_event_parse_key_event:
  * @event: A #GstEvent to inspect.
- * @key: A pointer to a location to receive the string identifying the key
- * press. The returned string is owned by the event, and valid only until the
- * event is unreffed.
+ * @key: (out) (optional) (transfer none): A pointer to a location to receive
+ *     the string identifying the key press. The returned string is owned by the
+ *     event, and valid only until the event is unreffed.
  */
 gboolean
 gst_navigation_event_parse_key_event (GstEvent * event, const gchar ** key)
@@ -761,12 +775,12 @@ gst_navigation_event_parse_key_event (GstEvent * event, const gchar ** key)
 /**
  * gst_navigation_event_parse_mouse_button_event:
  * @event: A #GstEvent to inspect.
- * @button: Pointer to a gint that will receive the button number associated
- * with the event.
- * @x: Pointer to a gdouble to receive the x coordinate of the mouse button
- * event.
- * @y: Pointer to a gdouble to receive the y coordinate of the mouse button
- * event.
+ * @button: (out) (optional): Pointer to a gint that will receive the button
+ *     number associated with the event.
+ * @x: (out) (optional): Pointer to a gdouble to receive the x coordinate of the
+ *     mouse button event.
+ * @y: (out) (optional): Pointer to a gdouble to receive the y coordinate of the
+ *     mouse button event.
  *
  * Retrieve the details of either a #GstNavigation mouse button press event or
  * a mouse button release event. Determine which type the event is using
@@ -803,8 +817,10 @@ gst_navigation_event_parse_mouse_button_event (GstEvent * event, gint * button,
 /**
  * gst_navigation_event_parse_mouse_move_event:
  * @event: A #GstEvent to inspect.
- * @x: Pointer to a gdouble to receive the x coordinate of the mouse movement.
- * @y: Pointer to a gdouble to receive the y coordinate of the mouse movement.
+ * @x: (out) (optional): Pointer to a gdouble to receive the x coordinate of the
+ *     mouse movement.
+ * @y: (out) (optional): Pointer to a gdouble to receive the y coordinate of the
+ *     mouse movement.
  *
  * Inspect a #GstNavigation mouse movement event and extract the coordinates
  * of the event.
@@ -832,11 +848,55 @@ gst_navigation_event_parse_mouse_move_event (GstEvent * event, gdouble * x,
   return ret;
 }
 
+/**
+ * gst_navigation_event_parse_mouse_scroll_event:
+ * @event: A #GstEvent to inspect.
+ * @x: (out) (optional): Pointer to a gdouble to receive the x coordinate of the
+ *     mouse movement.
+ * @y: (out) (optional): Pointer to a gdouble to receive the y coordinate of the
+ *     mouse movement.
+ * @delta_x: (out) (optional): Pointer to a gdouble to receive the delta_x coordinate of the
+ *     mouse movement.
+ * @delta_y: (out) (optional): Pointer to a gdouble to receive the delta_y coordinate of the
+ *     mouse movement.
+ *
+ * Inspect a #GstNavigation mouse scroll event and extract the coordinates
+ * of the event.
+ *
+ * Returns: TRUE if all coordinates could be extracted, otherwise FALSE.
+ *
+ * Since: 1.18
+ */
+gboolean
+gst_navigation_event_parse_mouse_scroll_event (GstEvent * event,
+    gdouble * x, gdouble * y, gdouble * delta_x, gdouble * delta_y)
+{
+  const GstStructure *s;
+  gboolean ret = TRUE;
+
+  g_return_val_if_fail (GST_NAVIGATION_EVENT_HAS_TYPE (event, MOUSE_SCROLL),
+      FALSE);
+
+  s = gst_event_get_structure (event);
+  if (x)
+    ret &= gst_structure_get_double (s, "pointer_x", x);
+  if (y)
+    ret &= gst_structure_get_double (s, "pointer_y", y);
+  if (delta_x)
+    ret &= gst_structure_get_double (s, "delta_pointer_x", delta_x);
+  if (delta_y)
+    ret &= gst_structure_get_double (s, "delta_pointer_y", delta_y);
+
+  WARN_IF_FAIL (ret, "Couldn't extract positions from mouse scroll event");
+
+  return ret;
+}
+
 /**
  * gst_navigation_event_parse_command:
  * @event: A #GstEvent to inspect.
- * @command: Pointer to GstNavigationCommand to receive the type of the
- * navigation event.
+ * @command: (out) (optional): Pointer to GstNavigationCommand to receive the
+ *     type of the navigation event.
  *
  * Inspect a #GstNavigation command event and retrieve the enum value of the
  * associated command.
