@@ -26,17 +26,6 @@
 #include <gst/base/gstbytereader.h>
 #include "video-anc.h"
 
-#if !GLIB_CHECK_VERSION(2, 47, 4)
-#ifdef __GNUC__
-#define G_GNUC_CHECK_VERSION(major, minor) \
-    ((__GNUC__ > (major)) || \
-     ((__GNUC__ == (major)) && \
-      (__GNUC_MINOR__ >= (minor))))
-#else
-#define G_GNUC_CHECK_VERSION(major, minor) 0
-#endif
-#endif
-
 /**
  * SECTION:gstvideoanc
  * @title: GstVideo Ancillary
@@ -932,7 +921,7 @@ gst_video_vbi_encoder_write_line (GstVideoVBIEncoder * encoder, guint8 * data)
 GType
 gst_video_caption_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
 
   if (g_once_init_enter (&type)) {
     static const gchar *tags[] = { NULL };
@@ -1042,7 +1031,7 @@ gst_buffer_add_video_caption_meta (GstBuffer * buffer,
   g_return_val_if_fail (meta != NULL, NULL);
 
   meta->caption_type = caption_type;
-  meta->data = g_memdup (data, size);
+  meta->data = g_memdup2 (data, size);
   meta->size = size;
 
   return meta;
@@ -1135,7 +1124,7 @@ gst_video_caption_type_to_caps (GstVideoCaptionType type)
 GType
 gst_video_afd_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
 
   if (g_once_init_enter (&type)) {
     static const gchar *tags[] = {
@@ -1162,6 +1151,53 @@ gst_video_afd_meta_init (GstMeta * meta, gpointer params, GstBuffer * buffer)
   return TRUE;
 }
 
+static gboolean
+gst_video_afd_meta_transform (GstBuffer * dest, GstMeta * meta,
+    GstBuffer * buffer, GQuark type, gpointer data)
+{
+  GstVideoAFDMeta *smeta = (GstVideoAFDMeta *) meta;
+
+  if (GST_META_TRANSFORM_IS_COPY (type)) {
+    GST_DEBUG ("copy AFD metadata");
+    gst_buffer_add_video_afd_meta (dest, smeta->field, smeta->spec, smeta->afd);
+    return TRUE;
+  } else if (GST_VIDEO_META_TRANSFORM_IS_SCALE (type)) {
+    GstVideoMetaTransform *trans = data;
+    gdouble diff;
+    gint ow, oh, nw, nh;
+    gint opn, opd, npn, npd;
+
+    ow = GST_VIDEO_INFO_WIDTH (trans->in_info);
+    nw = GST_VIDEO_INFO_WIDTH (trans->out_info);
+    oh = GST_VIDEO_INFO_HEIGHT (trans->in_info);
+    nh = GST_VIDEO_INFO_HEIGHT (trans->out_info);
+    opn = GST_VIDEO_INFO_PAR_N (trans->in_info);
+    opd = GST_VIDEO_INFO_PAR_D (trans->in_info);
+    npn = GST_VIDEO_INFO_PAR_N (trans->out_info);
+    npd = GST_VIDEO_INFO_PAR_D (trans->out_info);
+
+    /* if the aspect ratio stays the same we can copy the meta, otherwise
+     * we can't know if the aspect ratio was changed or black borders were
+     * introduced. Both would invalidate the AFD meta */
+
+    diff =
+        ABS (((gdouble) ow / (gdouble) oh) * ((gdouble) opn / (gdouble) opd) -
+        ((gdouble) nw / (gdouble) nh) * ((gdouble) npn / (gdouble) npd));
+    if (diff < 0.0001) {
+      GST_DEBUG ("copying AFD metadata, aspect ratio did not change");
+      gst_buffer_add_video_afd_meta (dest, smeta->field, smeta->spec,
+          smeta->afd);
+      return TRUE;
+    } else {
+      return FALSE;
+    }
+  } else {
+    /* return FALSE, if transform type is not supported */
+    return FALSE;
+  }
+
+}
+
 const GstMetaInfo *
 gst_video_afd_meta_get_info (void)
 {
@@ -1173,7 +1209,7 @@ gst_video_afd_meta_get_info (void)
         sizeof (GstVideoAFDMeta),
         gst_video_afd_meta_init,
         NULL,
-        NULL);
+        gst_video_afd_meta_transform);
     g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) mi);
   }
   return meta_info;
@@ -1227,7 +1263,7 @@ gst_buffer_add_video_afd_meta (GstBuffer * buffer, guint8 field,
 GType
 gst_video_bar_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
 
   if (g_once_init_enter (&type)) {
     static const gchar *tags[] = {
@@ -1255,6 +1291,22 @@ gst_video_bar_meta_init (GstMeta * meta, gpointer params, GstBuffer * buffer)
   return TRUE;
 }
 
+static gboolean
+gst_video_bar_meta_transform (GstBuffer * dest, GstMeta * meta,
+    GstBuffer * buffer, GQuark type, gpointer data)
+{
+  GstVideoBarMeta *smeta = (GstVideoBarMeta *) meta;
+
+  if (GST_META_TRANSFORM_IS_COPY (type)) {
+    GST_DEBUG ("copy Bar metadata");
+    gst_buffer_add_video_bar_meta (dest, smeta->field, smeta->is_letterbox,
+        smeta->bar_data1, smeta->bar_data2);
+    return TRUE;
+  } else {
+    /* return FALSE, if transform type is not supported */
+    return FALSE;
+  }
+}
 
 const GstMetaInfo *
 gst_video_bar_meta_get_info (void)
@@ -1267,7 +1319,7 @@ gst_video_bar_meta_get_info (void)
         sizeof (GstVideoBarMeta),
         gst_video_bar_meta_init,
         NULL,
-        NULL);
+        gst_video_bar_meta_transform);
     g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) mi);
   }
   return meta_info;
