@@ -47,7 +47,7 @@
  *
  */
 
-/* 
+/*
  * Formulas for PAR, DAR, width and height relations:
  *
  * dar_n   w   par_n
@@ -57,7 +57,7 @@
  * par_n    h   dar_n
  * ----- =  - * -----
  * par_d    w   dar_d
- * 
+ *
  *         dar_n   par_d
  * w = h * ----- * -----
  *         dar_d   par_n
@@ -161,7 +161,7 @@ static GstCaps *
 gst_video_scale_get_capslist (void)
 {
   static GstCaps *caps = NULL;
-  static volatile gsize inited = 0;
+  static gsize inited = 0;
 
   if (g_once_init_enter (&inited)) {
     caps = gst_static_caps_get (&gst_video_scale_format_caps);
@@ -195,7 +195,7 @@ static GstCaps *gst_video_scale_transform_caps (GstBaseTransform * trans,
 static GstCaps *gst_video_scale_fixate_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps, GstCaps * othercaps);
 static gboolean gst_video_scale_transform_meta (GstBaseTransform * trans,
-    GstBuffer * inbuf, GstMeta * meta, GstBuffer * outbuf);
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
 
 static gboolean gst_video_scale_set_info (GstVideoFilter * filter,
     GstCaps * in, GstVideoInfo * in_info, GstCaps * out,
@@ -210,6 +210,11 @@ static void gst_video_scale_get_property (GObject * object, guint prop_id,
 
 #define gst_video_scale_parent_class parent_class
 G_DEFINE_TYPE (GstVideoScale, gst_video_scale, GST_TYPE_VIDEO_FILTER);
+GST_ELEMENT_REGISTER_DEFINE (videoscale, "videoscale",
+    GST_RANK_NONE, GST_TYPE_VIDEO_SCALE);
+
+static GstCapsFeatures *features_format_interlaced,
+    *features_format_interlaced_sysmem;
 
 static void
 gst_video_scale_class_init (GstVideoScaleClass * klass)
@@ -470,8 +475,11 @@ gst_video_scale_transform_caps (GstBaseTransform * trans,
 
     /* If the features are non-sysmem we can only do passthrough */
     if (!gst_caps_features_is_any (features)
-        && gst_caps_features_is_equal (features,
-            GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY)) {
+        && (gst_caps_features_is_equal (features,
+                GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY)
+            || gst_caps_features_is_equal (features, features_format_interlaced)
+            || gst_caps_features_is_equal (features,
+                features_format_interlaced_sysmem))) {
       gst_structure_set (structure, "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
           "height", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
 
@@ -500,8 +508,8 @@ gst_video_scale_transform_caps (GstBaseTransform * trans,
 }
 
 static gboolean
-gst_video_scale_transform_meta (GstBaseTransform * trans, GstBuffer * inbuf,
-    GstMeta * meta, GstBuffer * outbuf)
+gst_video_scale_transform_meta (GstBaseTransform * trans, GstBuffer * outbuf,
+    GstMeta * meta, GstBuffer * inbuf)
 {
   GstVideoFilter *videofilter = GST_VIDEO_FILTER (trans);
   const GstMetaInfo *info = meta->info;
@@ -511,7 +519,8 @@ gst_video_scale_transform_meta (GstBaseTransform * trans, GstBuffer * inbuf,
   const gchar *const valid_tags[] = { GST_META_TAG_VIDEO_STR,
     GST_META_TAG_VIDEO_COLORSPACE_STR,
     GST_META_TAG_VIDEO_ORIENTATION_STR,
-    GST_META_TAG_VIDEO_SIZE_STR
+    GST_META_TAG_VIDEO_SIZE_STR,
+    NULL
   };
 
   tags = gst_meta_api_type_get_tags (info->api);
@@ -535,7 +544,7 @@ gst_video_scale_transform_meta (GstBaseTransform * trans, GstBuffer * inbuf,
   /* Cant handle the tags in this meta, let the parent class handle it */
   if (!should_copy) {
     return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (trans,
-        inbuf, meta, outbuf);
+        outbuf, meta, inbuf);
   }
 
   /* This meta is size sensitive, try to transform it accordingly */
@@ -1221,15 +1230,18 @@ gst_video_scale_src_event (GstBaseTransform * trans, GstEvent * event)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "videoscale", GST_RANK_NONE,
-          GST_TYPE_VIDEO_SCALE))
-    return FALSE;
+  features_format_interlaced =
+      gst_caps_features_new (GST_CAPS_FEATURE_FORMAT_INTERLACED, NULL);
+  features_format_interlaced_sysmem =
+      gst_caps_features_copy (features_format_interlaced);
+  gst_caps_features_add (features_format_interlaced_sysmem,
+      GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
 
   GST_DEBUG_CATEGORY_INIT (video_scale_debug, "videoscale", 0,
       "videoscale element");
   GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
 
-  return TRUE;
+  return GST_ELEMENT_REGISTER (videoscale, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
