@@ -24,6 +24,7 @@
 #include <gst/base/gstbasesink.h>
 
 #include <gst/check/gstcheck.h>
+#include <gst/check/gstharness.h>
 #include <string.h>
 
 /* kids, don't do this at home, skipping checks is *BAD* */
@@ -123,6 +124,8 @@ check_pad_template (GstPadTemplate * tmpl)
         case GST_VIDEO_FORMAT_GBR_10BE:
         case GST_VIDEO_FORMAT_GBR_10LE:
         case GST_VIDEO_FORMAT_NV12_64Z32:
+        case GST_VIDEO_FORMAT_NV12_4L4:
+        case GST_VIDEO_FORMAT_NV12_32L32:
           GST_LOG ("Ignoring lack of support for format %s", fmt_str);
           break;
         default:
@@ -499,89 +502,13 @@ CREATE_TEST (test_upscale_1x240_640x480_method_3, 3, 1, 240, 640, 480);
 
 #ifndef VSCALE_TEST_GROUP
 
-typedef struct
-{
-  gint width, height;
-  gint par_n, par_d;
-  gboolean ok;
-  GMainLoop *loop;
-} TestNegotiationData;
-
-static void
-_test_negotiation_message (GstBus * bus, GstMessage * message,
-    TestNegotiationData * data)
-{
-  GError *err = NULL;
-  gchar *debug;
-
-  switch (GST_MESSAGE_TYPE (message)) {
-    case GST_MESSAGE_ERROR:
-      gst_message_parse_error (message, &err, &debug);
-      gst_object_default_error (GST_MESSAGE_SRC (message), err, debug);
-      g_error_free (err);
-      g_free (debug);
-      g_assert_not_reached ();
-      break;
-    case GST_MESSAGE_WARNING:
-      gst_message_parse_warning (message, &err, &debug);
-      gst_object_default_error (GST_MESSAGE_SRC (message), err, debug);
-      g_error_free (err);
-      g_free (debug);
-      g_assert_not_reached ();
-      break;
-    case GST_MESSAGE_EOS:
-      g_main_loop_quit (data->loop);
-      break;
-    default:
-      break;
-  }
-}
-
-static void
-_test_negotiation_notify_caps (GObject * src, GParamSpec * pspec,
-    TestNegotiationData * data)
-{
-  GstCaps *caps;
-  GstStructure *s;
-  gint width, height;
-  gint par_n = 0, par_d = 0;
-
-  g_object_get (src, "caps", &caps, NULL);
-  if (caps == NULL)
-    return;
-
-  s = gst_caps_get_structure (caps, 0);
-
-  fail_unless (gst_structure_get_int (s, "width", &width));
-  fail_unless (gst_structure_get_int (s, "height", &height));
-  fail_unless (gst_structure_get_fraction (s, "pixel-aspect-ratio", &par_n,
-          &par_d) || (data->par_n == 1 && data->par_d == 1));
-
-  gst_caps_unref (caps);
-
-  fail_unless_equals_int (width, data->width);
-  fail_unless_equals_int (height, data->height);
-  if (par_n != 0 || par_d != 0) {
-    fail_unless_equals_int (par_n, data->par_n);
-    fail_unless_equals_int (par_d, data->par_d);
-  }
-
-  data->ok = (width == data->width) && (height == data->height)
-      && (par_n == data->par_n) && (par_d == data->par_d);
-
-  g_main_loop_quit (data->loop);
-}
-
 static void
 _test_negotiation (const gchar * src_templ, const gchar * sink_templ,
     gint width, gint height, gint par_n, gint par_d)
 {
   GstElement *pipeline;
   GstElement *src, *capsfilter1, *scale, *capsfilter2, *sink;
-  GstBus *bus;
-  GMainLoop *loop;
   GstCaps *caps;
-  TestNegotiationData data = { 0, 0, 0, 0, FALSE, NULL };
   GstPad *pad;
 
   GST_DEBUG ("Running test for src templ caps '%s' and sink templ caps '%s'",
@@ -592,7 +519,7 @@ _test_negotiation (const gchar * src_templ, const gchar * sink_templ,
 
   src = gst_element_factory_make ("videotestsrc", "src");
   fail_unless (src != NULL);
-  g_object_set (G_OBJECT (src), "num-buffers", 1, NULL);
+  g_object_set (G_OBJECT (src), "num-buffers", 1, "pattern", 2, NULL);
 
   capsfilter1 = gst_element_factory_make ("capsfilter", "filter1");
   fail_unless (capsfilter1 != NULL);
@@ -611,15 +538,8 @@ _test_negotiation (const gchar * src_templ, const gchar * sink_templ,
   g_object_set (G_OBJECT (capsfilter2), "caps", caps, NULL);
   gst_caps_unref (caps);
 
-  pad = gst_element_get_static_pad (capsfilter2, "sink");
-  fail_unless (pad != NULL);
-  g_signal_connect (pad, "notify::caps",
-      G_CALLBACK (_test_negotiation_notify_caps), &data);
-  gst_object_unref (pad);
-
   sink = gst_element_factory_make ("fakesink", "sink");
   fail_unless (sink != NULL);
-  g_object_set (sink, "async", FALSE, NULL);
 
   gst_bin_add_many (GST_BIN (pipeline), src, capsfilter1, scale, capsfilter2,
       sink, NULL);
@@ -633,37 +553,47 @@ _test_negotiation (const gchar * src_templ, const gchar * sink_templ,
   fail_unless (gst_element_link_pads_full (capsfilter2, "src", sink, "sink",
           LINK_CHECK_FLAGS));
 
-  loop = g_main_loop_new (NULL, FALSE);
+  fail_unless_equals_int (gst_element_set_state (pipeline, GST_STATE_PAUSED),
+      GST_STATE_CHANGE_ASYNC);
 
-  bus = gst_element_get_bus (pipeline);
-  fail_unless (bus != NULL);
-  gst_bus_add_signal_watch (bus);
+  /* Wait for pipeline to preroll, at which point negotiation is finished */
+  fail_unless_equals_int (gst_element_get_state (pipeline, NULL, NULL, -1),
+      GST_STATE_CHANGE_SUCCESS);
 
-  data.loop = loop;
-  data.width = width;
-  data.height = height;
-  data.par_n = par_n;
-  data.par_d = par_d;
-  data.ok = FALSE;
+  /* Get negotiated caps */
+  pad = gst_element_get_static_pad (capsfilter2, "sink");
+  fail_unless (pad != NULL);
+  caps = gst_pad_get_current_caps (pad);
+  fail_unless (caps != NULL);
+  gst_object_unref (pad);
 
-  g_signal_connect (bus, "message", G_CALLBACK (_test_negotiation_message),
-      &data);
+  /* Check negotiated caps */
+  {
+    gint out_par_n = 0, out_par_d = 0;
+    gint out_width, out_height;
+    GstStructure *s;
 
-  gst_bus_remove_signal_watch (bus);
-  gst_object_unref (bus);
+    s = gst_caps_get_structure (caps, 0);
 
-  fail_unless (gst_element_set_state (pipeline,
-          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS);
+    fail_unless (gst_structure_get_int (s, "width", &out_width));
+    fail_unless (gst_structure_get_int (s, "height", &out_height));
+    fail_unless (gst_structure_get_fraction (s, "pixel-aspect-ratio",
+            &out_par_n, &out_par_d) || (par_n == 1 && par_d == 1));
 
-  g_main_loop_run (loop);
-
-  fail_unless (data.ok == TRUE);
+    fail_unless_equals_int (width, out_width);
+    fail_unless_equals_int (height, out_height);
+    if (par_n != 0 || par_d != 0) {
+      fail_unless_equals_int (par_n, out_par_n);
+      fail_unless_equals_int (par_d, out_par_d);
+    }
+  }
+  gst_caps_unref (caps);
 
+  /* clean up */
   fail_unless (gst_element_set_state (pipeline,
           GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS);
 
   gst_object_unref (pipeline);
-  g_main_loop_unref (loop);
 }
 
 GST_START_TEST (test_negotiation)
@@ -706,6 +636,11 @@ GST_START_TEST (test_negotiation)
       ("video/x-raw,format=(string)AYUV,width=320,height=240,pixel-aspect-ratio=1/1",
       "video/x-raw,format=(string)AYUV,width=200,height=200,pixel-aspect-ratio=1/2",
       200, 200, 1, 2);
+
+  _test_negotiation
+      ("video/x-raw,format=(string)AYUV,width=854,height=480",
+      "video/x-raw,format=(string)AYUV,width=[2, 512, 2],height=[2, 512, 2],pixel-aspect-ratio=1/1",
+      512, 288, 1, 1);
 }
 
 GST_END_TEST;
@@ -842,6 +777,45 @@ gst_test_reverse_negotiation_sink_init (GstTestReverseNegotiationSink * sink)
   sink->nbuffers = 0;
 }
 
+GST_START_TEST (test_negotiate_alternate)
+{
+  GstHarness *h;
+  GstBuffer *buffer;
+  GstMapInfo map;
+
+  h = gst_harness_new ("videoscale");
+
+  buffer = gst_buffer_new_and_alloc (4);
+  gst_buffer_map (buffer, &map, GST_MAP_WRITE);
+  map.data[0] = 0x0;
+  map.data[1] = 0x0;
+  map.data[2] = 0x0;
+  map.data[3] = 0x0;
+  gst_buffer_unmap (buffer, &map);
+
+  gst_harness_set_sink_caps_str (h, "video/x-raw,width=1,height=1,format=ARGB");
+  gst_harness_set_src_caps_str (h,
+      "video/x-raw(format:Interlaced),interlace-mode=alternate,width=1,height=2,format=ARGB");
+  fail_unless_equals_int (gst_harness_push (h, gst_buffer_ref (buffer)),
+      GST_FLOW_NOT_NEGOTIATED);
+
+  gst_harness_set_sink_caps_str (h,
+      "video/x-raw(format:Interlaced),width=1,height=1,format=ARGB");
+  gst_harness_set_src_caps_str (h,
+      "video/x-raw,interlace-mode=alternate,width=1,height=2,format=ARGB");
+  fail_unless_equals_int (gst_harness_push (h, gst_buffer_ref (buffer)),
+      GST_FLOW_NOT_NEGOTIATED);
+
+  gst_harness_set_sink_caps_str (h,
+      "video/x-raw(format:Interlaced),interlace-mode=alternate,width=1,height=1,format=ARGB");
+  gst_harness_set_src_caps_str (h,
+      "video/x-raw(format:Interlaced),interlace-mode=alternate,width=1,height=2,format=ARGB");
+  fail_unless_equals_int (gst_harness_push (h, buffer), GST_FLOW_OK);
+
+  gst_harness_teardown (h);
+}
+
+GST_END_TEST;
 #if 0
 static void
 _test_reverse_negotiation_message (GstBus * bus, GstMessage * message,
@@ -1006,11 +980,13 @@ videoscale_suite (void)
   tcase_add_test (tc_chain, test_passthrough_method_2);
   tcase_add_test (tc_chain, test_passthrough_method_3);
   tcase_add_test (tc_chain, test_negotiation);
+  tcase_add_test (tc_chain, test_negotiate_alternate);
 #if 0
   tcase_add_test (tc_chain, test_reverse_negotiation);
 #endif
   tcase_add_test (tc_chain, test_basetransform_negotiation);
-#elif VSCALE_TEST_GROUP == 1
+#else
+#if VSCALE_TEST_GROUP == 1
   tcase_add_test (tc_chain, test_downscale_640x480_320x240_method_0);
   tcase_add_test (tc_chain, test_downscale_640x480_320x240_method_1);
   tcase_add_test (tc_chain, test_downscale_640x480_320x240_method_2);
@@ -1065,6 +1041,7 @@ videoscale_suite (void)
   tcase_add_test (tc_chain, test_upscale_1x240_640x480_method_2);
   tcase_add_test (tc_chain, test_upscale_1x240_640x480_method_3);
 #endif
+#endif /* VSCALE_TEST_GROUP */
 
   return s;
 }
